@@ -1,4 +1,4 @@
-# Copyright 2021 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -290,7 +290,8 @@ class RaspbianDevice(auxiliary_device.AuxiliaryDevice):
                        command_name="shell",
                        raise_error=False,
                        tries=1,
-                       port=0):
+                       port=0,
+                       timeout=None):
     """Sends a command, searches for a regex in the response, and returns a match group.
 
     Args:
@@ -303,6 +304,7 @@ class RaspbianDevice(auxiliary_device.AuxiliaryDevice):
         tries(int): how many times to try executing the command before
           failing.
         port(int): which port to send the shell command to.
+        timeout(float): Time in seconds to wait for device to respond.
 
     Returns:
         str: value of the capturing group with index 'regex_group' in the
@@ -321,7 +323,8 @@ class RaspbianDevice(auxiliary_device.AuxiliaryDevice):
         raise_error=raise_error,
         tries=tries,
         command_name=command_name,
-        port=port)
+        port=port,
+        timeout=timeout)
 
   @decorators.CapabilityDecorator(switchboard.SwitchboardDefault)
   def switchboard(self):
