@@ -1,4 +1,4 @@
-# Copyright 2021 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,10 +21,12 @@ from gazoo_device import detect_criteria
 from gazoo_device import gdm_logger
 from gazoo_device.auxiliary_devices import cambrionix
 from gazoo_device.auxiliary_devices import dli_powerswitch
+from gazoo_device.auxiliary_devices import dlink_switch
 from gazoo_device.auxiliary_devices import efr32
 from gazoo_device.auxiliary_devices import esp32
 from gazoo_device.auxiliary_devices import nrf52840
 from gazoo_device.auxiliary_devices import raspberry_pi
+from gazoo_device.auxiliary_devices import raspberry_pi_matter_controller
 from gazoo_device.auxiliary_devices import unifi_poe_switch
 from gazoo_device.auxiliary_devices import yepkit
 from gazoo_device.capabilities import comm_power_default
@@ -36,12 +38,14 @@ from gazoo_device.capabilities import file_transfer_adb
 from gazoo_device.capabilities import file_transfer_docker
 from gazoo_device.capabilities import file_transfer_echo
 from gazoo_device.capabilities import file_transfer_scp
+from gazoo_device.capabilities import flash_build_esptool
 from gazoo_device.capabilities import flash_build_jlink
+from gazoo_device.capabilities import matter_controller_chip_tool
+from gazoo_device.capabilities import matter_endpoints_accessor
 from gazoo_device.capabilities import package_management_android
 from gazoo_device.capabilities import pwrpc_button_default
 from gazoo_device.capabilities import pwrpc_common_default
-from gazoo_device.capabilities import pwrpc_light_default
-from gazoo_device.capabilities import pwrpc_lock_default
+from gazoo_device.capabilities import pwrpc_wifi_default
 from gazoo_device.capabilities import shell_ssh
 from gazoo_device.capabilities import switch_power_dli_powerswitch
 from gazoo_device.capabilities import switch_power_ethernet
@@ -57,16 +61,34 @@ from gazoo_device.capabilities.interfaces import event_parser_base
 from gazoo_device.capabilities.interfaces import fastboot_base
 from gazoo_device.capabilities.interfaces import file_transfer_base
 from gazoo_device.capabilities.interfaces import flash_build_base
+from gazoo_device.capabilities.interfaces import matter_controller_base
+from gazoo_device.capabilities.interfaces import matter_endpoints_base
 from gazoo_device.capabilities.interfaces import package_management_base
 from gazoo_device.capabilities.interfaces import pwrpc_button_base
 from gazoo_device.capabilities.interfaces import pwrpc_common_base
-from gazoo_device.capabilities.interfaces import pwrpc_light_base
-from gazoo_device.capabilities.interfaces import pwrpc_lock_base
+from gazoo_device.capabilities.interfaces import pwrpc_wifi_base
 from gazoo_device.capabilities.interfaces import shell_base
 from gazoo_device.capabilities.interfaces import switch_power_base
 from gazoo_device.capabilities.interfaces import switchboard_base
 from gazoo_device.capabilities.interfaces import usb_hub_base
+from gazoo_device.capabilities.matter_clusters import color_control_pw_rpc
+from gazoo_device.capabilities.matter_clusters import door_lock_pw_rpc
+from gazoo_device.capabilities.matter_clusters import level_control_pw_rpc
+from gazoo_device.capabilities.matter_clusters import on_off_pw_rpc
+from gazoo_device.capabilities.matter_clusters.interfaces import color_control_base
+from gazoo_device.capabilities.matter_clusters.interfaces import door_lock_base
+from gazoo_device.capabilities.matter_clusters.interfaces import level_control_base
+from gazoo_device.capabilities.matter_clusters.interfaces import on_off_base
+from gazoo_device.capabilities.matter_endpoints import color_temperature_light
+from gazoo_device.capabilities.matter_endpoints import dimmable_light
+from gazoo_device.capabilities.matter_endpoints import door_lock
+from gazoo_device.capabilities.matter_endpoints import on_off_light
+from gazoo_device.capabilities.matter_endpoints.interfaces import color_temperature_light_base
+from gazoo_device.capabilities.matter_endpoints.interfaces import dimmable_light_base
+from gazoo_device.capabilities.matter_endpoints.interfaces import door_lock_base as door_lock_endpoint_base
+from gazoo_device.capabilities.matter_endpoints.interfaces import on_off_light_base
 from gazoo_device.primary_devices import efr32_matter_lighting
+from gazoo_device.primary_devices import efr32_matter_locking
 from gazoo_device.primary_devices import esp32_matter_allclusters
 from gazoo_device.primary_devices import esp32_matter_locking
 from gazoo_device.primary_devices import nrf_matter_lighting
@@ -108,15 +130,18 @@ def export_extensions() -> Dict[str, Any]:
           esp32_matter_allclusters.Esp32MatterAllclusters,
           esp32_matter_locking.Esp32MatterLocking,
           efr32_matter_lighting.Efr32MatterLighting,
+          efr32_matter_locking.Efr32MatterLocking,
           nrf_matter_lighting.NrfMatterLighting,
       ],
       "auxiliary_devices": [
           cambrionix.Cambrionix,
           dli_powerswitch.DliPowerSwitch,
+          dlink_switch.DLinkSwitch,
           efr32.EFR32,
           esp32.ESP32,
           nrf52840.NRF52840,
           raspberry_pi.RaspberryPi,
+          raspberry_pi_matter_controller.RaspberryPiMatterController,
           unifi_poe_switch.UnifiPoeSwitch,
           yepkit.Yepkit,
       ],
@@ -127,34 +152,48 @@ def export_extensions() -> Dict[str, Any]:
           communication_types.JlinkSerialComms,
           communication_types.PigweedSerialComms,
           communication_types.PtyProcessComms,
-          communication_types.SshComms,
           communication_types.SerialComms,
+          communication_types.SnmpComms,
+          communication_types.SshComms,
           communication_types.UsbComms,
           communication_types.YepkitComms,
       ],
-      "detect_criteria":
-          detect_criteria.DETECT_CRITERIA,
+      "detect_criteria": detect_criteria.DETECT_CRITERIA,
       "capability_interfaces": [
+          color_control_base.ColorControlClusterBase,
+          color_temperature_light_base.ColorTemperatureLightBase,
           comm_power_base.CommPowerBase,
           device_power_base.DevicePowerBase,
+          dimmable_light_base.DimmableLightBase,
+          door_lock_endpoint_base.DoorLockBase,
+          door_lock_base.DoorLockClusterBase,
           embedded_script_base.EmbeddedScriptBase,
           event_parser_base.EventParserBase,
           fastboot_base.FastbootBase,
           file_transfer_base.FileTransferBase,
           flash_build_base.FlashBuildBase,
+          level_control_base.LevelControlClusterBase,
+          matter_controller_base.MatterControllerBase,
+          matter_endpoints_base.MatterEndpointsBase,
+          on_off_base.OnOffClusterBase,
+          on_off_light_base.OnOffLightBase,
           package_management_base.PackageManagementBase,
           pwrpc_button_base.PwRPCButtonBase,
           pwrpc_common_base.PwRPCCommonBase,
-          pwrpc_light_base.PwRPCLightBase,
-          pwrpc_lock_base.PwRPCLockBase,
+          pwrpc_wifi_base.PwRPCWifiBase,
           shell_base.ShellBase,
           switchboard_base.SwitchboardBase,
           switch_power_base.SwitchPowerBase,
           usb_hub_base.UsbHubBase,
       ],
       "capability_flavors": [
+          color_control_pw_rpc.ColorControlClusterPwRpc,
+          color_temperature_light.ColorTemperatureLightEndpoint,
           comm_power_default.CommPowerDefault,
           device_power_default.DevicePowerDefault,
+          dimmable_light.DimmableLightEndpoint,
+          door_lock.DoorLockEndpoint,
+          door_lock_pw_rpc.DoorLockClusterPwRpc,
           embedded_script_dli_powerswitch.EmbeddedScriptDliPowerswitch,
           event_parser_default.EventParserDefault,
           fastboot_default.FastbootDefault,
@@ -162,12 +201,17 @@ def export_extensions() -> Dict[str, Any]:
           file_transfer_docker.FileTransferDocker,
           file_transfer_echo.FileTransferEcho,
           file_transfer_scp.FileTransferScp,
+          flash_build_esptool.FlashBuildEsptool,
           flash_build_jlink.FlashBuildJLink,
+          level_control_pw_rpc.LevelControlClusterPwRpc,
+          matter_controller_chip_tool.MatterControllerChipTool,
+          matter_endpoints_accessor.MatterEndpointsAccessor,
+          on_off_light.OnOffLightEndpoint,
+          on_off_pw_rpc.OnOffClusterPwRpc,
           package_management_android.PackageManagementAndroid,
           pwrpc_button_default.PwRPCButtonDefault,
           pwrpc_common_default.PwRPCCommonDefault,
-          pwrpc_light_default.PwRPCLightDefault,
-          pwrpc_lock_default.PwRPCLockDefault,
+          pwrpc_wifi_default.PwRPCWifiDefault,
           shell_ssh.ShellSSH,
           switch_power_dli_powerswitch.SwitchPowerDliPowerswitch,
           switch_power_ethernet.SwitchPowerEthernet,
